@@ -1,6 +1,6 @@
 package Catalyst::Request;
 
-use IO::Socket qw[AF_INET inet_aton];
+use Socket qw( getaddrinfo getnameinfo AI_NUMERICHOST NI_NAMEREQD NIx_NOSERV );
 use Carp;
 use utf8;
 use URI::http;
@@ -123,7 +123,7 @@ sub _build_body_data {
 
     # Not sure if these returns should not be exceptions...
     my $content_type = $self->content_type || return;
-    return unless ($self->method eq 'POST' || $self->method eq 'PUT');
+    return unless ($self->method eq 'POST' || $self->method eq 'PUT' || $self->method eq 'PATCH' || $self->method eq 'DELETE');
 
     my ($match) = grep { $content_type =~/$_/i }
       keys(%{$self->data_handlers});
@@ -132,14 +132,17 @@ sub _build_body_data {
       my $fh = $self->body;
       local $_ = $fh;
       return $self->data_handlers->{$match}->($fh, $self);
-    } else { 
-      Catalyst::Exception->throw("$content_type is does not have an available data handler");
+    } else {
+      Catalyst::Exception->throw(
+        sprintf '%s does not have an available data handler. Valid data_handlers are %s.',
+          $content_type, join ', ', sort keys %{$self->data_handlers}
+      );
     }
 }
 
 has _use_hash_multivalue => (
-    is=>'ro', 
-    required=>1, 
+    is=>'ro',
+    required=>1,
     default=> sub {0});
 
 # Amount of data to read from input on each pass
@@ -248,7 +251,7 @@ sub prepare_body {
     my ( $self ) = @_;
 
     # If previously applied middleware created the HTTP::Body object, then we
-    # just use that one.  
+    # just use that one.
 
     if(my $plack_body = $self->_has_env ? $self->env->{'plack.request.http.body'} : undef) {
         $self->_body($plack_body);
@@ -280,7 +283,7 @@ sub prepare_body {
     # Ok if we get this far, we have to read psgi.input into the new body
     # object.  Lets play nice with any plack app or other downstream, so
     # we create a buffer unless one exists.
-     
+
     my $stream_buffer;
     if ($self->env->{'psgix.input.buffered'}) {
         # Be paranoid about previous psgi middleware or apps that read the
@@ -432,11 +435,30 @@ sub body {
 
 has hostname => (
   is        => 'rw',
-  required  => 1,
   lazy      => 1,
   default   => sub {
     my ($self) = @_;
-    gethostbyaddr( inet_aton( $self->address ), AF_INET ) || $self->address
+    my ( $err, $sockaddr ) = getaddrinfo(
+        $self->address,
+        # no service
+        '',
+        { flags => AI_NUMERICHOST }
+    );
+    if ( $err ) {
+        $self->_log->warn("resolve of hostname failed: $err");
+        return $self->address;
+    }
+    ( $err, my $hostname ) = getnameinfo(
+        $sockaddr->{addr},
+        NI_NAMEREQD,
+        # we are only interested in the hostname, not the servicename
+        NIx_NOSERV
+    );
+    if ( $err ) {
+        $self->_log->warn("resolve of hostname failed: $err");
+        return $self->address;
+    }
+    return $hostname;
   },
 );
 
@@ -549,17 +571,19 @@ C<multipart/form-data>, in which case a L<File::Temp> object is returned.
 
 =head2 $req->body_data
 
-Returns a Perl representation of POST/PUT body data that is not classic HTML
+Returns a Perl representation of body data that is not classic HTML
 form data, such as JSON, XML, etc.  By default, Catalyst will parse incoming
-data of the type 'application/json' and return access to that data via this
-method.  You may define addition data_handlers via a global configuration
+data of the type 'application/json' for POST, PUT, PATCH or DELETE methods,
+and return access to that data via this method.
+
+You may define addition data_handlers via a global configuration
 setting.  See L<Catalyst\DATA HANDLERS> for more information.
 
-If the POST is malformed in some way (such as undefined or not content that
+If the body is malformed in some way (such as undefined or not content that
 matches the content-type) we raise a L<Catalyst::Exception> with the error
 text as the message.
 
-If the POSTed content type does not match an available data handler, this
+If the body content type does not match an available data handler, this
 will also raise an exception.
 
 =head2 $req->body_parameters
@@ -771,7 +795,7 @@ sub param {
 
     # If anything in @_ is undef, carp about that, and remove it from
     # the list;
-    
+
     my @params = grep { defined($_) ? 1 : do {carp "You called ->params with an undefined value"; 0} } @_;
 
     if ( @params == 1 ) {
@@ -1122,7 +1146,7 @@ If parameters have already been set will clear the parameters and build them aga
 
 =head2 $self->env
 
-Access to the raw PSGI env.  
+Access to the raw PSGI env.
 
 =head2 meta
 
